import 'dart:convert';
import 'dart:ui' as ui;

import '../models/login-attempts.dart';
import '../services/login_attempts.dart';
import 'package:ecom_widgets/utils/validators.dart';
import 'package:ecom_widgets/animation/ScaleRoute.dart';
import 'package:ecom_widgets/widgets/custom_text_field.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:fluttertoast/fluttertoast.dart';

import 'phone_number_widget.dart';

class PhoneNumberOtpConfirmPage extends StatelessWidget {
  TextEditingController otpCodeController = TextEditingController();
  final ConfirmationResult confirmationResult;
  bool confirmingOTP = false;

  final String phoneNumber;
  final _formKey = GlobalKey<FormState>();

  PhoneNumberOtpConfirmPage(
      {Key? key, required this.confirmationResult, required this.phoneNumber})
      : super(key: key);
  @override
  Widget build(BuildContext context) {
    String defaultFontFamily = 'Roboto-Light.ttf';
    double defaultFontSize = 14;
    double defaultIconSize = 17;

    confirmOtpRequest() async {
      if (_formKey.currentState != null && !_formKey.currentState!.validate())
        return;

      if (confirmingOTP)
        return;
      else
        confirmingOTP = true;

      if (otpCodeController.text.length != 6) {
        Fluttertoast.showToast(
          msg: "Error: Invalid OTP",
          backgroundColor: Colors.red,
        );
      } else {
        UserCredential userCredential = await confirmationResult
            .confirm(otpCodeController.text)
            .whenComplete(
              () => Navigator.pop(context),
            )
            .catchError(
          (onError) async {
            LoginAttempts? attempt = await LoginAttemptService()
                .attemptLogin(LoginAttempts.fromJson({
              'phone_number': phoneNumber,
            }));

            Fluttertoast.showToast(
              msg: "Error: Invalid OTP. Try signing in again. $phoneNumber",
              backgroundColor: Colors.red,
            );
          },
        );
      }

      confirmingOTP = false;
    }

    return Scaffold(
      resizeToAvoidBottomInset: true,
      body: Center(
        child: Container(
          padding: EdgeInsets.only(left: 20, right: 20, top: 35, bottom: 30),
          width: 500,
          color: Colors.white70,
          child: SingleChildScrollView(
            child: Form(
              key: _formKey,
              child: Column(
                children: <Widget>[
                  InkWell(
                    child: Container(
                      child: Align(
                        alignment: Alignment.topLeft,
                        child: Icon(Icons.close),
                      ),
                    ),
                    onTap: () {
                      Navigator.pushReplacementNamed(context, '/');
                    },
                  ),
                  Container(
                    width: 230,
                    height: 100,
                    alignment: Alignment.center,
                    child: Image.asset(
                      PhoneNumberSignInWidget.assetLink,
                    ),
                  ),
                  SizedBox(
                    height: 50,
                  ),
                  CustomTextFormField(
                    hintText: "Enter OTP",
                    prefixIcon: Icons.phone,
                    controller: otpCodeController,
                    validator: numberValidator,
                    keyboardType: TextInputType.number,
                    inputFormatters: [FilteringTextInputFormatter.digitsOnly],
                    maxLengthEnforcement: MaxLengthEnforcement.enforced,
                    maxLength: 6,
                  ),
                  SizedBox(
                    height: 15,
                  ),
                  SizedBox(
                    height: 15,
                  ),
                  SignInButtonWidget(
                    onPressed: confirmOtpRequest,
                  ),
                  SizedBox(
                    height: 2,
                  ),
                  SizedBox(
                    height: 30,
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      Container(
                        decoration: BoxDecoration(
                          gradient: new LinearGradient(
                              colors: [
                                Colors.black12,
                                Colors.black54,
                              ],
                              begin: const FractionalOffset(0.0, 0.0),
                              end: const FractionalOffset(1.0, 1.0),
                              stops: [0.0, 1.0],
                              tileMode: TileMode.clamp),
                        ),
                        width: 100.0,
                        height: 1.0,
                      ),
                      Padding(
                        padding: EdgeInsets.only(left: 15.0, right: 15.0),
                        child: Text(
                          "Or",
                          style: TextStyle(
                            color: Color(0xFF2c2b2b),
                            fontSize: 16.0,
                            fontFamily: "WorkSansMedium",
                          ),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                          gradient: new LinearGradient(
                              colors: [
                                Colors.black54,
                                Colors.black12,
                              ],
                              begin: const FractionalOffset(0.0, 0.0),
                              end: const FractionalOffset(1.0, 1.0),
                              stops: [0.0, 1.0],
                              tileMode: TileMode.clamp),
                        ),
                        width: 100.0,
                        height: 1.0,
                      ),
                    ],
                  ),
                  SizedBox(
                    height: 30,
                  ),
                  Align(
                    alignment: Alignment.bottomCenter,
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: <Widget>[
                        Container(
                          child: Text(
                            "Don't have an account? ",
                            style: TextStyle(
                              color: Color(0xFF666666),
                              fontFamily: defaultFontFamily,
                              fontSize: defaultFontSize,
                              fontStyle: FontStyle.normal,
                            ),
                          ),
                        ),
                        InkWell(
                          onTap: () {
                            Navigator.pop(context);
                            Navigator.pushNamed(
                              context,
                              '/register',
                            );
                          },
                          child: Container(
                            child: Text(
                              "Sign Up",
                              style: TextStyle(
                                color: Theme.of(context).primaryColor,
                                fontFamily: defaultFontFamily,
                                fontSize: defaultFontSize,
                                fontStyle: FontStyle.normal,
                              ),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }
}

class SignInButtonWidget extends StatelessWidget {
  final Function onPressed;

  const SignInButtonWidget({Key? key, required this.onPressed})
      : super(key: key);
  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      decoration: new BoxDecoration(
        borderRadius: BorderRadius.all(Radius.circular(5.0)),
        boxShadow: <BoxShadow>[
          BoxShadow(
            color: Theme.of(context).primaryColor,
          ),
          BoxShadow(
            color: Theme.of(context).primaryColor,
          ),
        ],
        gradient: new LinearGradient(
            colors: [
              Theme.of(context).primaryColor,
              Theme.of(context).accentColor
            ],
            begin: const FractionalOffset(0.2, 0.2),
            end: const FractionalOffset(1.0, 1.0),
            stops: [0.0, 1.0],
            tileMode: TileMode.clamp),
      ),
      child: MaterialButton(
          highlightColor: Colors.transparent,
          splashColor: Theme.of(context).primaryColor,
          //shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(5.0))),
          child: Padding(
            padding:
                const EdgeInsets.symmetric(vertical: 10.0, horizontal: 42.0),
            child: Text(
              "CONFIRM OTP",
              style: TextStyle(
                  color: Colors.white,
                  fontSize: 25.0,
                  fontFamily: "WorkSansBold"),
            ),
          ),
          onPressed: () => {onPressed()}),
    );
  }
}
