//conditional import
import '../models/login-attempts.dart';
import '../services/login_attempts.dart';
import '../services/user_service.dart';
import 'package:ecom_widgets/utils/validators.dart';
import 'package:ecom_widgets/animation/ScaleRoute.dart';
import 'package:ecom_widgets/widgets/custom_text_field.dart';
import 'package:ecom_widgets/widgets/styled_button.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/services.dart';
import 'package:fluttertoast/fluttertoast.dart';

import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'phone_number_widget.dart';

class PhoneNumberAndroidWidget extends PhoneNumberSignInWidget {
  TextEditingController phoneNumberController = TextEditingController();
  TextEditingController otpController = TextEditingController();
  bool sendingOTP = false;
  final _formKey = GlobalKey<FormState>();

  @override
  Widget build(BuildContext context) {
    String defaultFontFamily = 'Roboto-Light.ttf';
    double defaultFontSize = 14;
    double defaultIconSize = 17;

    sendOtpAndAttemptConfirmation() async {
      if (_formKey.currentState != null && !_formKey.currentState!.validate())
        return;

      if (sendingOTP)
        return;
      else
        sendingOTP = true;

      if (phoneNumberController.text.length != 10) {
        Fluttertoast.showToast(
          msg: "Error: Phone number invalid",
          backgroundColor: Colors.red,
          //gravity: ToastGravity.,
        );
      } else {
        bool phoneNumberExists = await UserService()
            .checkUserPhoneExists(phoneNumberController.text);
        if (phoneNumberExists != true) {
          Fluttertoast.showToast(
            msg: "Error: No user with Phone number found",
            backgroundColor: Colors.red,
          );
          sendingOTP = false;
          return;
        }

        LoginAttempts? attempt =
            await LoginAttemptService().getLogin(LoginAttempts.fromJson({
          'phone_number': phoneNumberController.text,
        }));

        if (attempt?.banned ?? false) {
          Fluttertoast.showToast(
            msg:
                "Error: Phone number banned for 5 hours. Please try again later.",
            backgroundColor: Colors.red,
            //gravity: ToastGravity.,
          );

          sendingOTP = false;
          return;
        }

        await FirebaseAuth.instance.verifyPhoneNumber(
          phoneNumber: '+91 ${phoneNumberController.text}',
          verificationCompleted: (PhoneAuthCredential credential) async {
            sendingOTP = false;
            print("verificationCompleted");
            await FirebaseAuth.instance.signInWithCredential(credential);

            Navigator.pop(context);
          },
          verificationFailed: (FirebaseAuthException e) {
            print("verificationFailed: ${e}");
          },
          codeSent: (String verificationId, int? resendToken) async {
            String smsCode = 'xxxx';
            print("codeSent");
            final _dialogFormKey = GlobalKey<FormState>();

            await showDialog(
                context: context,
                builder: (BuildContext context) {
                  return AlertDialog(
                    content: Container(
                      height: 200,
                      child: Form(
                        key: _dialogFormKey,
                        child: Stack(
                          children: [
                            Column(
                              children: [
                                SizedBox(
                                  height: 30,
                                ),
                                CustomTextFormField(
                                  hintText: "Enter OTP",
                                  controller: otpController,
                                  validator: numberValidator,

                                  // Phone number
                                  keyboardType: TextInputType.number,
                                  inputFormatters: [
                                    FilteringTextInputFormatter.digitsOnly
                                  ],
                                  maxLengthEnforcement:
                                      MaxLengthEnforcement.enforced,
                                  maxLength: 6,
                                ),
                                SizedBox(
                                  height: 30,
                                ),
                                StyledButtonWidget(
                                  onPressed: () {
                                    if (_dialogFormKey.currentState != null &&
                                        !_dialogFormKey.currentState!
                                            .validate()) return;

                                    smsCode = otpController.text;
                                    Navigator.pushReplacementNamed(
                                        context, '/');
                                  },
                                  text: "VERIFY OTP",
                                  fontSize: 20,
                                ),
                              ],
                            )
                          ],
                        ),
                      ),
                    ),
                  );
                });

            PhoneAuthCredential credential = PhoneAuthProvider.credential(
              verificationId: verificationId,
              smsCode: smsCode,
            );

            FirebaseAuth.instance
                .signInWithCredential(credential)
                .whenComplete(
                  () => Navigator.pop(context),
                )
                .catchError((e) async {
              LoginAttempts? attempt = await LoginAttemptService()
                  .attemptLogin(LoginAttempts.fromJson({
                'phone_number': phoneNumberController.text,
              }));
            });

            // Navigator.pushReplacementNamed(context, '/');
          },
          codeAutoRetrievalTimeout: (String verificationId) {
            print("codeAutoRetrievalTimeout");
          },
        );
      }
      sendingOTP = false;
    }

    return Scaffold(
      resizeToAvoidBottomInset: true,
      body: Center(
        child: Container(
          padding: EdgeInsets.only(left: 20, right: 20, top: 35, bottom: 30),
          width: 500,
          //height: MediaQuery.of(context).size.height,
          color: Colors.white70,
          child: SingleChildScrollView(
            child: Form(
              key: _formKey,
              child: Column(
                children: <Widget>[
                  InkWell(
                    child: Container(
                      child: Align(
                        alignment: Alignment.topLeft,
                        child: Icon(Icons.close),
                      ),
                    ),
                    onTap: () {
                      Navigator.pop(context);
                    },
                  ),
                  Container(
                    width: 230,
                    height: 100,
                    alignment: Alignment.center,
                    child: Image.asset(
                      PhoneNumberSignInWidget.assetLink,
                    ),
                  ),
                  SizedBox(
                    height: 50,
                  ),
                  CustomTextFormField(
                    hintText: "Phone Number",
                    prefixIcon: Icons.phone,
                    controller: phoneNumberController,

                    // Phone number settings
                    validator: phoneNumberValidator,
                    keyboardType: TextInputType.number,
                    inputFormatters: [FilteringTextInputFormatter.digitsOnly],
                    maxLengthEnforcement: MaxLengthEnforcement.enforced,
                    maxLength: 10,
                  ),
                  SizedBox(
                    height: 15,
                  ),
                  // _customTextField(
                  //   hintText: "Password",
                  //   prefixIcon: Icons.lock_outline,
                  //   suffixIcon: Icons.remove_red_eye,
                  // ),
                  // SizedBox(
                  //   height: 15,
                  // ),
                  // Container(
                  //   width: double.infinity,
                  //   child: Text(
                  //     "Forgot your password?",
                  //     style: TextStyle(
                  //       color: Color(0xFF666666),
                  //       fontFamily: defaultFontFamily,
                  //       fontSize: defaultFontSize,
                  //       fontStyle: FontStyle.normal,
                  //     ),
                  //     textAlign: TextAlign.end,
                  //   ),
                  // ),
                  SizedBox(
                    height: 15,
                  ),
                  StyledButtonWidget(
                    onPressed: sendOtpAndAttemptConfirmation,
                    text: "SEND OTP",
                  ),
                  SizedBox(
                    height: 2,
                  ),
                  SizedBox(
                    height: 30,
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      Container(
                        decoration: BoxDecoration(
                          gradient: new LinearGradient(
                              colors: [
                                Colors.black12,
                                Colors.black54,
                              ],
                              begin: const FractionalOffset(0.0, 0.0),
                              end: const FractionalOffset(1.0, 1.0),
                              stops: [0.0, 1.0],
                              tileMode: TileMode.clamp),
                        ),
                        width: 100.0,
                        height: 1.0,
                      ),
                      Padding(
                        padding: EdgeInsets.only(left: 15.0, right: 15.0),
                        child: Text(
                          "Or",
                          style: TextStyle(
                              color: Color(0xFF2c2b2b),
                              fontSize: 16.0,
                              fontFamily: "WorkSansMedium"),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                          gradient: new LinearGradient(
                              colors: [
                                Colors.black54,
                                Colors.black12,
                              ],
                              begin: const FractionalOffset(0.0, 0.0),
                              end: const FractionalOffset(1.0, 1.0),
                              stops: [0.0, 1.0],
                              tileMode: TileMode.clamp),
                        ),
                        width: 100.0,
                        height: 1.0,
                      ),
                    ],
                  ),
                  // FacebookGoogleLogin(),
                  SizedBox(
                    height: 30,
                  ),
                  Align(
                    alignment: Alignment.bottomCenter,
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: <Widget>[
                        Container(
                          child: Text(
                            "Don't have an account? ",
                            style: TextStyle(
                              color: Color(0xFF666666),
                              fontFamily: defaultFontFamily,
                              fontSize: defaultFontSize,
                              fontStyle: FontStyle.normal,
                            ),
                          ),
                        ),
                        InkWell(
                          onTap: () {
                            Navigator.pop(context);
                            Navigator.pushNamed(context, '/register');
                          },
                          child: Container(
                            child: Text(
                              "Sign Up",
                              style: TextStyle(
                                color: Theme.of(context).primaryColor,
                                fontFamily: defaultFontFamily,
                                fontSize: defaultFontSize,
                                fontStyle: FontStyle.normal,
                              ),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }
}

PhoneNumberSignInWidget getPhoneNumberSignInWidget() =>
    PhoneNumberAndroidWidget();
