import 'dart:convert';
import 'dart:html';
import 'dart:ui' as ui;
import 'dart:js' as js;
//conditional import
import '../models/login-attempts.dart';
import '../services/login_attempts.dart';
import '../services/user_service.dart';
import 'package:ecom_widgets/utils/validators.dart';
import 'package:ecom_widgets/animation/ScaleRoute.dart';
import 'package:ecom_widgets/widgets/custom_text_field.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:fluttertoast/fluttertoast.dart';

import 'PhoneNumberOtpPage.dart';
import 'phone_number_widget.dart';

class PhoneNumberWebWidget extends PhoneNumberSignInWidget {
  TextEditingController phoneNumberController = TextEditingController();
  bool sendingOTP = false;
  final _formKey = GlobalKey<FormState>();
  // RecaptchaVerifier recapcha;

  @override
  Widget build(BuildContext context) {
    String defaultFontFamily = 'Roboto-Light.ttf';
    double defaultFontSize = 14;
    double defaultIconSize = 17;

    phoneSignInRequest() async {
      if (_formKey.currentState != null && !_formKey.currentState!.validate()) {
        return;
      }

      if (sendingOTP)
        return;
      else
        sendingOTP = true;

      if (phoneNumberController.text.length != 10) {
        Fluttertoast.showToast(
          msg: "Error: Phone number invalid",
          backgroundColor: Colors.red,
          //gravity: ToastGravity.,
        );
      } else {
        // recapcha = RecaptchaVerifier(
        //   container: 'recaptcha',
        //   size: RecaptchaVerifierSize.compact,
        //   theme: RecaptchaVerifierTheme.dark,
        // );

        bool phoneNumberExists = await UserService()
            .checkUserPhoneExists(phoneNumberController.text);
        if (phoneNumberExists != true) {
          Fluttertoast.showToast(
            msg: "Error: No user with Phone number found",
            backgroundColor: Colors.red,
          );
          sendingOTP = false;
          return;
        }

        LoginAttempts? attempt =
            await LoginAttemptService().getLogin(LoginAttempts.fromJson({
          'phone_number': phoneNumberController.text,
        }));

        if (attempt?.banned ?? false) {
          Fluttertoast.showToast(
            msg:
                "Error: Phone number banned for 5 hours. Please try again later.",
            backgroundColor: Colors.red,
          );

          sendingOTP = false;
          return;
        }
        print('phoneNumberController.text: ' + phoneNumberController.text);
        ConfirmationResult confirmationResult =
            await FirebaseAuth.instance.signInWithPhoneNumber(
          '+91 ${phoneNumberController.text}',
        );

        // recapcha.clear();

        final el = window.document.getElementById('__ff-recaptcha-container');
        if (el != null) {
          el.style.visibility = 'hidden';
          el.remove();
        }

        Navigator.pop(context);
        Navigator.push(
          context,
          ScaleRoute(
            page: PhoneNumberOtpConfirmPage(
              phoneNumber: phoneNumberController.text,
              confirmationResult: confirmationResult,
            ),
          ),
        );
      }
      sendingOTP = false;
    }

    return Scaffold(
      resizeToAvoidBottomInset: true,
      body: Center(
        child: Container(
          padding: EdgeInsets.only(left: 20, right: 20, top: 35, bottom: 30),
          width: 500,
          //height: MediaQuery.of(context).size.height,
          color: Colors.white70,
          child: SingleChildScrollView(
            child: Form(
              key: _formKey,
              child: Column(
                children: <Widget>[
                  InkWell(
                    child: Container(
                      child: Align(
                        alignment: Alignment.topLeft,
                        child: Icon(Icons.close),
                      ),
                    ),
                    onTap: () {
                      Navigator.pop(context);
                    },
                  ),
                  Container(
                    width: 230,
                    height: 100,
                    alignment: Alignment.center,
                    child: Image.asset(
                      PhoneNumberSignInWidget.assetLink,
                    ),
                  ),
                  SizedBox(
                    height: 50,
                  ),
                  CustomTextFormField(
                    // labelText: "Phone Number",
                    hintText: "Phone Number",
                    prefixIcon: Icons.phone,
                    controller: phoneNumberController,

                    // Phone number
                    validator: phoneNumberValidator,
                    keyboardType: TextInputType.number,
                    inputFormatters: [FilteringTextInputFormatter.digitsOnly],
                    maxLengthEnforcement: MaxLengthEnforcement.enforced,
                    maxLength: 10,
                  ),
                  SizedBox(
                    height: 15,
                  ),
                  // _customTextField(
                  //   hintText: "Password",
                  //   prefixIcon: Icons.lock_outline,
                  //   suffixIcon: Icons.remove_red_eye,
                  // ),
                  // SizedBox(
                  //   height: 15,
                  // ),
                  // Container(
                  //   width: double.infinity,
                  //   child: Text(
                  //     "Forgot your password?",
                  //     style: TextStyle(
                  //       color: Color(0xFF666666),
                  //       fontFamily: defaultFontFamily,
                  //       fontSize: defaultFontSize,
                  //       fontStyle: FontStyle.normal,
                  //     ),
                  //     textAlign: TextAlign.end,
                  //   ),
                  // ),
                  SizedBox(
                    height: 15,
                  ),
                  SignInButtonWidget(
                    onPressed: phoneSignInRequest,
                  ),
                  SizedBox(
                    height: 2,
                  ),
                  SizedBox(
                    height: 30,
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      Container(
                        decoration: BoxDecoration(
                          gradient: new LinearGradient(
                              colors: [
                                Colors.black12,
                                Colors.black54,
                              ],
                              begin: const FractionalOffset(0.0, 0.0),
                              end: const FractionalOffset(1.0, 1.0),
                              stops: [0.0, 1.0],
                              tileMode: TileMode.clamp),
                        ),
                        width: 100.0,
                        height: 1.0,
                      ),
                      Padding(
                        padding: EdgeInsets.only(left: 15.0, right: 15.0),
                        child: Text(
                          "Or",
                          style: TextStyle(
                              color: Color(0xFF2c2b2b),
                              fontSize: 16.0,
                              fontFamily: "WorkSansMedium"),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                          gradient: new LinearGradient(
                              colors: [
                                Colors.black54,
                                Colors.black12,
                              ],
                              begin: const FractionalOffset(0.0, 0.0),
                              end: const FractionalOffset(1.0, 1.0),
                              stops: [0.0, 1.0],
                              tileMode: TileMode.clamp),
                        ),
                        width: 100.0,
                        height: 1.0,
                      ),
                    ],
                  ),
                  // FacebookGoogleLogin(),
                  SizedBox(
                    height: 30,
                  ),
                  Align(
                    alignment: Alignment.bottomCenter,
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: <Widget>[
                        Container(
                          child: Text(
                            "Don't have an account? ",
                            style: TextStyle(
                              color: Color(0xFF666666),
                              fontFamily: defaultFontFamily,
                              fontSize: defaultFontSize,
                              fontStyle: FontStyle.normal,
                            ),
                          ),
                        ),
                        InkWell(
                          onTap: () {
                            Navigator.pop(context);
                            Navigator.pushNamed(
                              context,
                              '/register',
                            );
                          },
                          child: Container(
                            child: Text(
                              "Sign Up",
                              style: TextStyle(
                                color: Theme.of(context).primaryColor,
                                fontFamily: defaultFontFamily,
                                fontSize: defaultFontSize,
                                fontStyle: FontStyle.normal,
                              ),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }
}

class SignInButtonWidget extends StatelessWidget {
  final Function onPressed;

  const SignInButtonWidget({Key? key, required this.onPressed})
      : super(key: key);
  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      decoration: new BoxDecoration(
        borderRadius: BorderRadius.all(Radius.circular(5.0)),
        boxShadow: <BoxShadow>[
          BoxShadow(
            color: Theme.of(context).primaryColor,
          ),
          BoxShadow(
            color: Theme.of(context).primaryColor,
          ),
        ],
        gradient: new LinearGradient(
            colors: [
              Theme.of(context).primaryColor,
              Theme.of(context).accentColor
            ],
            begin: const FractionalOffset(0.2, 0.2),
            end: const FractionalOffset(1.0, 1.0),
            stops: [0.0, 1.0],
            tileMode: TileMode.clamp),
      ),
      child: MaterialButton(
          highlightColor: Colors.transparent,
          splashColor: Theme.of(context).primaryColor,
          //shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(5.0))),
          child: Padding(
            padding:
                const EdgeInsets.symmetric(vertical: 10.0, horizontal: 42.0),
            child: Text(
              "REQUEST OTP",
              style: TextStyle(
                  color: Colors.white,
                  fontSize: 25.0,
                  fontFamily: "WorkSansBold"),
            ),
          ),
          onPressed: () => {onPressed()}),
    );
  }
}

PhoneNumberSignInWidget getPhoneNumberSignInWidget() => PhoneNumberWebWidget();
